% function  [rLoss0, Eout, rList, lossFunc, sWave] = FCavAnal( T1, R, L, A, Ns, rWhk, Nhk )
% calculate round trip loss, rLoss0 (ppm), and loss function, lossFunc and
% spartial wave length, sWave, for the lossFunc, at Ns points.
% rWhk is the ratio of the FFT window to the mirror radius A, default = 2.
% Nhk is the number of grid points in A*rW
% Eout contains fields for the case without aberration.
% See HankelFP about the structure of Eout
%
%{
% example

% fill R, L and A for the case you want
R = [1.0000    1.0000    1.0000    1.0000   81.5619   44.6655   25.7747 14.0926];
L = [1.9997    1.9990    1.9970    1.9900   81.6435   44.7102   25.8005 14.1067];
A = [0.0421    0.0231    0.0133    0.0073    0.2083    0.1541    0.1171 0.0866];
bs = [0.0053    0.0039    0.0030    0.0022    0.0053    0.0039    0.0030 0.0022];
ic = 3;
Ns = 10;
rWhk = 2;
Nhk = 512;
[rLoss0, Eout, rList, lossFunc, sWave] = ...
FCavAnal( 400e-6, R(ic), L(ic), A(ic), Ns, rWhk, Nhk );

rLoss0

subplot(2,1,1)
semilogy( rList.^2, abs(Eout.toITM.E).^2, 'x-', ...
rList.^2, abs(Eout.toITM.E(1)).^2*exp( -2*rList.^2/beamsize(ic)^2 ) );
grid
line( [beamsize(ic)^2 beamsize(ic)^2], [1 1e10])
axis([0 (4*beamsize(ic))^2 1 1e10])
subplot(2,1,2)
semilogx( sWave, lossFunc, 'x-' );
grid
%}

function  [rLoss0, Eout, rList, lossFunc, sWave] = FCavAnal( T1, ROC, L, A, Ns, rWhk, Nhk )

if nargin < 7
    Nhk = 128;
end
if nargin < 6
    rWhk = 2;
end
if nargin < 5
    Ns = 10;
end

Whk = A * rWhk;
rList = HankelTK(Whk, Nhk).rList;

% test mass power transmittance
cavity.ITM.T = T1;
cavity.ETM.T = 0;
% arm loss. Make it 70ppm when no abberation is explicitly added.
cavity.armIE.Loss = 0;
% 1/ROC
cavity.ITM.invROC = 1/ROC;
cavity.ETM.invROC = 1/ROC;
% mirror radius
cavity.ITM.radius = A;
cavity.ETM.radius = A;
% cavity length
cavity.armIE.L = L;

% ETM surface aberration
if Ns == 0
    lossFunc = zeros(0,0);
    sWave = zeros(0,0);
else
    lamSmin = Whk / Nhk * 2;
    lamSmax = A / 2;

    sWave = exp( linspace(log(lamSmin), log(lamSmax), Ns ) );

    amp = 1e-10;
    for is = 1:Ns

        delETM = amp * cos( 2*pi*rList / sWave(is) );

        [Eout, Ein, lossFunc(is), cavVal] = ...
            HankelFP( cavity, Nhk, Whk, 0, 0, delETM, 1, 0 );

        if ~cavVal.locked
            warning( 'lock failed for %d th spatial wavelength of %g mm', is, sWave(is)*1000 );
            lossFunc(is) = NaN;
        end
    
    end
% rms of amp * cos() is amp^2 / 2, 
% so to calculate loss for rms = 1nm^2, divide by 2/(amp/1nm)^2
    lossFunc = lossFunc * 2 /(amp/1e-9)^2 * 1e6;
    sWave = sWave * 1000;
end

% calculate roundtrip loss without aberration
[Eout, Ein, rLoss0] = ...
HankelFP( cavity, Nhk, Whk, 0, 0, 0, 1, 0 );
rLoss0 = rLoss0 * 1e6;

