%
% classdef HankelTK
%   Hankel toolkit for cavity simulation using Hankel transformation
%
%
% May 15, 2010 Hiro Yamamoto  hiro@ligo.caltech.edu
%
classdef HankelTK < handle
    properties (SetObservable = true)
        a = 0.3;            % window size. half of FFT window
        N = 256;            % number of samplings
        accuracy = 1e-6;    % accuracy of zeta values
    end
    
    properties (SetAccess = protected)
        zetaList            % zeros of J0 bessel
        rList               % r sampling values
        HPlus               % x space to f space conversion matrix
        J0Norm              % normalization factors used for dot product
    end
    
%     properties (Hidden = true)
%         initDone = false;   % set to true after constructor
%     end
    
    % implementation of methods
    methods
        % scalar product
        function fg = hkDotProd( obj, f, g )
            fg = 2*pi*sum( conj(f) .* g ./ obj.J0Norm );
        end

        % constructors
        function obj = HankelTK( a__, N__, acc__ )
            % use arguments to change the settings if specified
            if nargin >= 3
                obj.accuracy = acc__;
            end
            if nargin >= 2
                obj.N = N__;
            end
            if nargin >= 1
                obj.a = a__;
            end

            % update all table values
            updateAll( obj );
            
            % set listerner so that table values are updated automatically
            addlistener(obj,'a','PostSet',@obj.updateAll);
            addlistener(obj,'N','PostSet',@obj.updateAll);
            addlistener(obj,'accuracy','PostSet',@obj.updateAll);

        end
        
        % recalculate all table values
        % this is used as listener
        function updateAll( obj, src, evnt )
            % calculate zetas
            obj.zetaList = HankelTK.updateZetas( obj.N, obj.accuracy );

            % r sampling values
            obj.rList = obj.a / obj.zetaList(end) * obj.zetaList;

            % x->f matrix and notmalization factor
            calcHPlus( obj );
        end

        % x->f coversion, normalization
        function calcHPlus( obj )
            obj.HPlus = zeros( obj.N, obj.N );
            obj.J0Norm = ...
                (obj.zetaList(end) * ...
                besselj(0, obj.zetaList)/obj.a).^2 / 2;
            for ix = 1 : obj.N
                obj.HPlus(ix,:) = besselj(0, ...
                    obj.zetaList(ix)/obj.zetaList(end)*obj.zetaList) ...
                    ./ obj.J0Norm;
            end
        end

    end % end of methods
    
    methods (Static)
        
        % calculate N values of zeros of J1 bessel
        function zetas = updateZetas( N, accuracy )
            zetas = zeros(N,1);
            for in = 1 : N
                zetas(in) = HankelTK.getZetaK( in-1, accuracy );
            end
        end
        
        % k'th zero of J1
        function zeta = getZetaK( k, accuracy )
            if k <= 0
                zeta = 0;
            else
                zeta = ( k + 0.25 ) * pi;
                delzval = 0.01;
                while delzval > accuracy
                    rat10 = besselj(1,zeta) / besselj(0, zeta);
                    zetaNew = zeta * (1 - rat10/(zeta - rat10) );
                    delzval = abs( zetaNew - zeta );
                    zeta = zetaNew;
                end
            end
        end
        
    end % of methods (static)
end
